// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2003 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
// ==============================================================

#include "orbitersdk.h"

const double PB_FUELMASS = 750.0;
const double PB_ISP = 5e4;
const double PB_MAXMAINTH = 3e4;
const double PB_MAXHOVERTH = 1.5e4;
const double PB_MAXRCSTH = 2e2;

// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
double LiftCoeff (double aoa)
{
	const int nlift = 9;
	static const double AOA[nlift] = {-180*RAD,-60*RAD,-30*RAD,-1*RAD,15*RAD,20*RAD,25*RAD,60*RAD,180*RAD};
	static const double CL[nlift]  = {       0,      0,   -0.1,     0,   0.2,  0.25,   0.2,     0,      0};
	static const double SCL[nlift] = {(CL[1]-CL[0])/(AOA[1]-AOA[0]), (CL[2]-CL[1])/(AOA[2]-AOA[1]),
		                              (CL[3]-CL[2])/(AOA[3]-AOA[2]), (CL[4]-CL[3])/(AOA[4]-AOA[3]),
									  (CL[5]-CL[4])/(AOA[5]-AOA[4]), (CL[6]-CL[5])/(AOA[6]-AOA[5]),
									  (CL[7]-CL[6])/(AOA[7]-AOA[6]), (CL[8]-CL[7])/(AOA[8]-AOA[7])};
	for (int i = 0; i < nlift-1 && AOA[i+1] < aoa; i++);
	return CL[i] + (aoa-AOA[i])*SCL[i];
}

// ==============================================================
// API interface
// ==============================================================

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete vessel;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	THRUSTER_HANDLE th, th_rcs[12], th_group[4];

	// physical specs
	vessel->SetSize (3.5);
	vessel->SetEmptyMass (500.0);
	//vessel->SetCOG_elev (1.5);
	vessel->SetCW (0.3, 0.3, 0.6, 0.9);
	vessel->SetWingAspect (0.7);
	vessel->SetWingEffectiveness (2.5);
	vessel->SetCrossSections (_V(10.5,15.0,5.8));
	vessel->SetRotDrag (_V(0.6,0.6,0.35));
	if (vessel->GetFlightModel() >= 1) {
		vessel->SetPitchMomentScale (1e-4);
		vessel->SetBankMomentScale (1e-4);
	}
	vessel->SetPMI (_V(2.28,2.31,0.79));
	vessel->SetTrimScale (0.05);
	vessel->SetCameraOffset (_V(0,0.8,0));
	vessel->SetLiftCoeffFunc (LiftCoeff);
	vessel->SetDockParams (_V(0,1.3,-1), _V(0,1,0), _V(0,0,-1));
	vessel->SetTouchdownPoints (_V(0,-1.5,2), _V(-1,-1.5,-1.5), _V(1,-1.5,-1.5));

	// propellant resources
	PROPELLANT_HANDLE hpr = vessel->CreatePropellantResource (PB_FUELMASS);

	// thruster defintions
	th = vessel->CreateThruster (_V(0,0,-4.35), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	vessel->CreateThrusterGroup (&th, 1, THGROUP_MAIN);
	vessel->AddExhaust (th, 8, 1, _V(0,0.3,-4.35), _V(0,0,-1));

	th = vessel->CreateThruster (_V(0,-1.5,0), _V(0,1,0), PB_MAXHOVERTH, hpr, PB_ISP);
	vessel->CreateThrusterGroup (&th, 1, THGROUP_HOVER);
	vessel->AddExhaust (th, 8, 1, _V(0,-1.5,1), _V(0,-1,0));
	vessel->AddExhaust (th, 8, 1, _V(0,-1.5,-1), _V(0,-1,0));

	th_rcs[ 0] = vessel->CreateThruster (_V( 1,0, 3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 1] = vessel->CreateThruster (_V( 1,0, 3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 2] = vessel->CreateThruster (_V(-1,0, 3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 3] = vessel->CreateThruster (_V(-1,0, 3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 4] = vessel->CreateThruster (_V( 1,0,-3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 5] = vessel->CreateThruster (_V( 1,0,-3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 6] = vessel->CreateThruster (_V(-1,0,-3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 7] = vessel->CreateThruster (_V(-1,0,-3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 8] = vessel->CreateThruster (_V( 1,0, 3), _V(-1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 9] = vessel->CreateThruster (_V(-1,0, 3), _V( 1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[10] = vessel->CreateThruster (_V( 1,0,-3), _V(-1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[11] = vessel->CreateThruster (_V(-1,0,-3), _V( 1,0,0), PB_MAXRCSTH, hpr, PB_ISP);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	th_group[2] = th_rcs[5];
	th_group[3] = th_rcs[7];
	vessel->CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	th_group[2] = th_rcs[4];
	th_group[3] = th_rcs[6];
	vessel->CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	vessel->CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	vessel->CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	vessel->CreateThrusterGroup (th_group, 4, THGROUP_ATT_UP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	vessel->CreateThrusterGroup (th_group, 4, THGROUP_ATT_DOWN);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[11];
	vessel->CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	vessel->CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	vessel->CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	vessel->CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);

	// visual specs
	vessel->AddMesh ("ShuttlePB");
}

#ifdef UNDEF
// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	// physical specs
	vessel->SetSize (3.5);
	vessel->SetEmptyMass (500.0/*2000.0*/);
	vessel->SetMaxFuelMass (750.0/*3000.0*/);
	vessel->SetISP (5e4);
	vessel->SetMaxThrust (ENGINE_MAIN, 1.2e5);
	vessel->SetMaxThrust (ENGINE_RETRO, 4e4);
	vessel->SetMaxThrust (ENGINE_HOVER, 8e4);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 1e3);
	vessel->SetCOG_elev (1.5);
	vessel->SetCW (0.3, 0.3, 0.6, 0.9);
	vessel->SetWingAspect (0.7);
	vessel->SetWingEffectiveness (2.5);
	vessel->SetCrossSections (_V(10.5,15.0,5.8));
	vessel->SetRotDrag (_V(0.6,0.6,0.35));
	if (vessel->GetFlightModel() >= 1) {
		vessel->SetPitchMomentScale (1e-4);
		vessel->SetBankMomentScale (1e-4);
	}
	vessel->SetPMI (_V(2.28,2.31,0.79));
	vessel->SetTrimScale (0.05);
	vessel->SetCameraOffset (_V(0,0.8,0));
	vessel->SetLiftCoeffFunc (LiftCoeff);
	vessel->SetDockParams (_V(0,1.3,-1), _V(0,1,0), _V(0,0,-1));
	vessel->SetTouchdownPoints (_V(0,-1.5,2), _V(-1,-1.5,-1.5), _V(1,-1.5,-1.5));

	// visual specs
	UINT idx;
	vessel->AddMesh ("ShuttlePB");
	vessel->AddExhaustRef (EXHAUST_MAIN, _V(0,0.3,-4.35), 8, 1, NULL);
	vessel->AddExhaustRef (EXHAUST_RETRO, _V(0,-0.75, 3), 4, 1, NULL);
	vessel->AddExhaustRef (EXHAUST_HOVER, _V(0,-1.5,-1), 8, 1, NULL);
	vessel->AddExhaustRef (EXHAUST_HOVER, _V(0,-1.5, 1), 8, 1, NULL);
	idx = vessel->AddAttExhaustRef (_V(0,-1,2), _V(0,-1,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 0, 0);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 1, 0);
	idx = vessel->AddAttExhaustRef (_V(0,1.3,-1.95), _V(0,1,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 0, 0);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 1, 1);
	idx = vessel->AddAttExhaustRef (_V(0,1.3,-0.05), _V(0,1,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 0, 1);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 1, 1);
	idx = vessel->AddAttExhaustRef (_V(0,-1,-1.95), _V(0,-1,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 0, 1);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 1, 0);
	idx = vessel->AddAttExhaustRef (_V(-1.5,-0.6,-2), _V(-1,0,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 1, 0);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 0, 0);
	idx = vessel->AddAttExhaustRef (_V(0.55,-0.6,2.75), _V(1,0,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 1, 0);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 0, 1);
	idx = vessel->AddAttExhaustRef (_V(1.5,-0.6,-2), _V(1,0,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 1, 1);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 0, 1);
	idx = vessel->AddAttExhaustRef (_V(-0.55,-0.6,2.75), _V(-1,0,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 1, 1);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 0, 0);
	idx = vessel->AddAttExhaustRef (_V(-1.2,-1,-0.9), _V(0,-1,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 2, 0);
	idx = vessel->AddAttExhaustRef (_V(0.93,1.3,-0.9), _V(0,1,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 2, 0);
	idx = vessel->AddAttExhaustRef (_V(-0.93, 1.3,-0.9), _V(0,1,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 2, 1);
	idx = vessel->AddAttExhaustRef (_V(1.2,-1,-0.9), _V(0,-1,0), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_ROT, 2, 1);
	idx = vessel->AddAttExhaustRef (_V(0.3,-0.75,3), _V(0,0,1), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 2, 1);
	idx = vessel->AddAttExhaustRef (_V(-0.3,-0.75,3), _V(0,0,1), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 2, 1);
	idx = vessel->AddAttExhaustRef (_V(1,0.3,-3), _V(0,0,-1), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 2, 0);
	idx = vessel->AddAttExhaustRef (_V(-1,0.3,-3), _V(0,0,-1), 0.5, 0.5);
	vessel->AddAttExhaustMode (idx, ATTMODE_LIN, 2, 0);
}
#endif

